@@ -69,6 +69,9 @@ void      Scan_Number           (StmInf *pstm,Bool integer_only);
 static
 void      Scan_Quoted           (StmInf *pstm);
 
+static
+int       Scan_Quoted_Char      (StmInf *pstm,Bool convert,int c0);
+
 
 
 
@@ -303,7 +306,7 @@ static void Scan_Number(StmInf *pstm,Bool integer_only)
 
 {
  int   lg;
- int   radix;
+ int   radix,i;
  char *p,*f;
  int   c_orig0;
 
@@ -326,9 +329,25 @@ static void Scan_Number(StmInf *pstm,Bool integer_only)
  if (lg!=1 || token.int_num!=0 || strchr("'bBoOxX",c)==NULL)
      goto push_back;
 
- if (c=='\'')
+ if (c=='\'')                                             /* 0'<character> */
     {
-     Read_Next_Char(pstm,TRUE);
+     c=Scan_Quoted_Char(pstm,TRUE,'\'');
+     if (c== -1)                                       /* <character> is ' */
+        {
+         token.line=pstm->line_count+1;
+         token.col =pstm->line_pos+1;
+         err_msg="quote character expected here";
+        }
+
+     if (c== -2 || c== -3)
+        {
+         Unget_Last_Char
+
+         token.line=pstm->line_count+1;
+         token.col =pstm->line_pos+1;
+         err_msg="character expected here";
+        }
+
      token.int_num=c;
      return;
     }
@@ -338,13 +357,15 @@ static void Scan_Number(StmInf *pstm,Bool integer_only)
                           : (f="0123456789abcdefABCDEF",16);
  p=token.name;
  Read_Next_Char(pstm,TRUE);
- while(strchr(f,c)!=NULL)
+ token.int_num=0;
+ while((p=strchr(f,c))!=NULL)
     {
-     *p++=c;
+     i=p-f;
+     if (i>=16)
+         i-=6;
+     token.int_num=token.int_num*radix+i;
      Read_Next_Char(pstm,TRUE);
     }
- *p='\0';
- token.int_num=strtol(token.name,NULL,radix);
  goto push_back;
 
 
@@ -402,11 +423,8 @@ static void Scan_Quoted(StmInf *pstm)
 
 {
  int   c0;
- int   radix;
  char *s;
- char *p,*f;
  Bool  convert=(c_orig!='\'');
- int   x;
 
  token.type=(c_type==QT) ? TOKEN_NAME
                          : (c_type==DQ) ? TOKEN_STRING
@@ -416,91 +434,22 @@ static void Scan_Quoted(StmInf *pstm)
 
  for(;;)
     {
-     Read_Next_Char(pstm,convert);
-     if (c==c0)
+     c=Scan_Quoted_Char(pstm,convert,c0);
+     if (c== -1)
         {
-         if (Scan_Peek_Char(pstm,convert)!=c0)           /* '' or "" or `` */
-            {
-             *s='\0';
-             return;
-            }
-
-         Read_Next_Char(pstm,convert);
-         *s++ = c;
-         continue;
+         *s='\0';
+         return;
         }
 
-     if (c!='\\')
-        {
-         if (c==EOF || c=='\n')
-             goto term_error;
+     if (c== -2)                                              /* EOF or \n */
+         break;
 
-         *s++ = c;
-         continue;
-        }
-                                                        /* escape sequence */
-     Read_Next_Char(pstm,convert);
-     if (c=='\n')                                      /* \ followed by \n */
+     if (c== -3)                                       /* \ followed by \n */
          continue;
 
-     if (strchr("\\'\"`",c))                     /* \\ or \' or \" or \`  "*/
-        {
-         *s++ =c;
-         continue;
-        }
-
-     if ((p=(char *) strchr(escape_symbol,c)))     /* \a \b \f \n \r \t \v */
-        {
-         *s++ =escape_char[p-escape_symbol];
-         continue;
-        }
-
-     if (c=='x' || ('0'<=c && c<='7'))                       /* \xnn\ \nn\ */
-        {
-         p=s;
-         radix=(c=='x') ? (f="0123456789abcdefABCDEF", 16)
-                        : (*p++=c, f="01234567",        8);
-
-         Read_Next_Char(pstm,convert);
-         while(strchr(f,c)!=NULL)
-             {
-              *p++=c;
-              Read_Next_Char(pstm,convert);
-             }
-         *p='\0';
-
-         x=strtol(s,NULL,radix);
-         if (!Is_Valid_Code(x) && err_msg==NULL)
-            {
-             token.line=pstm->line_count+1;
-             token.col =pstm->line_pos;
-             err_msg="invalid character code in \\constant\\ sequence";
-            }
-         *s++ =(unsigned char) x;
-         if (c!='\\')
-            {
-             if (err_msg==NULL)
-                {
-                 token.line=pstm->line_count+1;
-                 token.col =pstm->line_pos;
-                 err_msg="\\ expected in \\constant\\ sequence";
-                }
-
-             Unget_Last_Char
-            }
-
-         continue;
-        }
-
-     if (err_msg==NULL)
-        {
-         token.line=pstm->line_count+1;
-         token.col =pstm->line_pos;
-         err_msg="unknown escape sequence";
-        }
+     *s++=c;
     }
-
-term_error:
+                                                                  /* error */
  *s='\0';
 
  if (err_msg)
@@ -532,6 +481,102 @@ term_error:
 
 
 
+/*-------------------------------------------------------------------------*/
+/* SCAN_QUOTED_CHAR                                                        */
+/*                                                                         */
+/*-------------------------------------------------------------------------*/
+static int Scan_Quoted_Char(StmInf *pstm,Bool convert,int c0)
+
+{
+ int   radix;
+ char *p,*f;
+ int   x,i;
+
+ Read_Next_Char(pstm,convert);
+ if (c==c0)
+    {
+     if (Scan_Peek_Char(pstm,convert)!=c0)               /* '' or "" or `` */
+         return -1;
+
+     Read_Next_Char(pstm,convert);
+     return c;
+    }
+
+ if (c==EOF || c=='\n')
+     return -2;
+
+ if (c!='\\')
+     return c;
+                                                        /* escape sequence */
+ Read_Next_Char(pstm,convert);
+ if (c=='\n')                                          /* \ followed by \n */
+     return -3;
+
+ if (strchr("\\'\"`",c))                         /* \\ or \' or \" or \`  "*/
+     return c;
+
+ if ((p=(char *) strchr(escape_symbol,c)))         /* \a \b \f \n \r \t \v */
+     return escape_char[p-escape_symbol];
+
+ if (c=='x' || ('0'<=c && c<='7'))                           /* \xnn\ \nn\ */
+    {
+     if (c=='x')
+        {
+         radix=16;
+         f="0123456789abcdefABCDEF";
+         x=0;
+        }
+      else
+        {
+         radix=8;
+         f="01234567";
+         x=c-'0';
+        }
+
+     Read_Next_Char(pstm,convert);
+     while((p=strchr(f,c))!=NULL)
+        {
+         i=p-f;
+         if (i>=16)
+             i-=6;
+         x=x*radix+i;
+         Read_Next_Char(pstm,convert);
+        }
+
+     if (!Is_Valid_Code(x) && err_msg==NULL)
+        {
+         token.line=pstm->line_count+1;
+         token.col =pstm->line_pos;
+         err_msg="invalid character code in \\constant\\ sequence";
+        }
+     if (c!='\\')
+        {
+         if (err_msg==NULL)
+            {
+             token.line=pstm->line_count+1;
+             token.col =pstm->line_pos;
+             err_msg="\\ expected in \\constant\\ sequence";
+            }
+
+         Unget_Last_Char
+        }
+
+     return (int) (unsigned char) x;
+    }
+
+ if (err_msg==NULL)
+    {
+     token.line=pstm->line_count+1;
+     token.col =pstm->line_pos;
+     err_msg="unknown escape sequence";
+    }
+
+ return 0;
+}
+
+
+
+
 /*-------------------------------------------------------------------------*/
 /* RECOVER_AFTER_ERROR                                                     */
 /*                                                                         */
