@@ -336,7 +336,11 @@ static void SIGSEGV_Handler(int sig,int code,struct sigcontext *scp)
 #elif defined(M_ix86_linux) || defined(M_powerpc_linux)
 #include <asm/sigcontext.h>
 
+#if 0   /* old linux */
 static void SIGSEGV_Handler(int sig,struct sigcontext_struct scp)
+#else
+static void SIGSEGV_Handler(int sig,struct sigcontext scp)
+#endif
 
 #else
 
