@@ -107,7 +107,7 @@ one_token(C,Token,C2):-
                    ; 
          C >= 0'A, C =< 0'Z, C1 is C+0'a-0'A
                    ;
-         C = 0'', C1=C),
+         C = 0'\', C1=C),
 	!,
 	unget_code(stream_fd,C1),
 	read_atom(stream_fd,Token1),
