@@ -645,17 +645,22 @@ static int Scanner(int complex_atom)
 
          if ((*p++=*cur_line_p++)=='\\')
             {
-             if (*cur_line_p=='x')
-                {
-                 cur_line_p++;
-                 i=16;
-                }
+             if (strchr("abfnrtv",*cur_line_p))      /* \a \b \f \n \r \t \v */
+                 *p++=*cur_line_p++;
               else
-                 i=8;
-             i=strtol(cur_line_p,&p1,i);            /* stop on the closing \ */
-             cur_line_p=p1+1;
-             sprintf(p,"%03o",i);
-             p+=3;
+                {
+                 if (*cur_line_p=='x')
+                    {
+                     cur_line_p++;
+                     i=16;
+                    }
+                  else
+                     i=8;
+                 i=strtol(cur_line_p,&p1,i);        /* stop on the closing \ */
+                 cur_line_p=p1+1;
+                 sprintf(p,"%03o",i);
+                 p+=3;
+                }
             }
         }
 
